@@ -2,7 +2,7 @@ from __future__ import with_statement
 import subprocess
 import functools
 from contextlib import contextmanager
-from plumbum.commands.processes import run_proc
+from plumbum.commands.processes import run_proc, iter_lines
 from plumbum.lib import six
 from tempfile import TemporaryFile
 from subprocess import PIPE, Popen
@@ -92,12 +92,14 @@ class BaseCommand(object):
     def _get_encoding(self):
         raise NotImplementedError()
 
-    def setenv(self, **envvars):
-        """Creates a BoundEnvCommand with the given environment variables"""
+    def with_env(self, **envvars):
+        """Returns a BoundEnvCommand with the given environment variables"""
         if not envvars:
             return self
         return BoundEnvCommand(self, envvars)
 
+    setenv = with_env
+
     @property
     def machine(self):
         raise NotImplementedError()
@@ -175,6 +177,7 @@ class BaseCommand(object):
             try:
                 return run_proc(p, retcode, timeout)
             finally:
+                del p.run  # to break cyclic reference p -> cell -> p
                 for f in [p.stdin, p.stdout, p.stderr]:
                     try:
                         f.close()
@@ -280,14 +283,15 @@ class Pipeline(BaseCommand):
         if srcproc.stdin:
             srcproc.stdin.close()
         dstproc.srcproc = srcproc
-        
+
         # monkey-patch .wait() to wait on srcproc as well (it's expected to die when dstproc dies)
         dstproc_wait = dstproc.wait
         @functools.wraps(Popen.wait)
         def wait2(*args, **kwargs):
-            rc = dstproc_wait(*args, **kwargs)
-            srcproc.wait(*args, **kwargs)
-            return rc
+            rc_dst = dstproc_wait(*args, **kwargs)
+            rc_src = srcproc.wait(*args, **kwargs)
+            dstproc.returncode = rc_src or rc_dst
+            return dstproc.returncode
         dstproc.wait = wait2
         return dstproc
 
@@ -404,10 +408,6 @@ class ConcreteCommand(BaseCommand):
     def _get_encoding(self):
         return self.encoding
 
-    @property
-    def machine(self):
-        return self.cmd.machine
-
     def formulate(self, level = 0, args = ()):
         argv = [str(self.executable)]
         for a in args:
