@@ -1,11 +1,13 @@
 import logging
 import errno
+import stat
 import socket
 from plumbum.machines.remote import BaseRemoteMachine
 from plumbum.machines.session import ShellSession
 from plumbum.lib import _setdoc, six
 from plumbum.path.local import LocalPath
-from plumbum.path.remote import RemotePath
+from plumbum.path.remote import RemotePath, StatRes
+from plumbum.commands.processes import iter_lines
 try:
     # Sigh... we need to gracefully-import paramiko for Sphinx builds, etc
     import paramiko
@@ -95,6 +97,12 @@ class ParamikoPopen(object):
         stderr = six.b("").join(six.b(s) for s in stderr)
         return stdout, stderr
 
+    def iter_lines(self, timeout=None, **kwargs):
+        if timeout is not None:
+            raise NotImplementedError("The 'timeout' parameter is not supported with ParamikoMachine")
+        return iter_lines(self, _iter_lines=_iter_lines, **kwargs)
+
+    __iter__ = iter_lines
 
 class ParamikoMachine(BaseRemoteMachine):
     """
@@ -135,9 +143,24 @@ class ParamikoMachine(BaseRemoteMachine):
 
     :param connect_timeout: timeout for TCP connection
     """
+
+    class RemoteCommand(BaseRemoteMachine.RemoteCommand):
+        def __or__(self, *_):
+            raise NotImplementedError("Not supported with ParamikoMachine")
+        def __gt__(self, *_):
+            raise NotImplementedError("Not supported with ParamikoMachine")
+        def __rshift__(self, *_):
+            raise NotImplementedError("Not supported with ParamikoMachine")
+        def __ge__(self, *_):
+            raise NotImplementedError("Not supported with ParamikoMachine")
+        def __lt__(self, *_):
+            raise NotImplementedError("Not supported with ParamikoMachine")
+        def __lshift__(self, *_):
+            raise NotImplementedError("Not supported with ParamikoMachine")
+
     def __init__(self, host, user = None, port = None, password = None, keyfile = None,
             load_system_host_keys = True, missing_host_policy = None, encoding = "utf8",
-            look_for_keys = None, connect_timeout = None):
+            look_for_keys = None, connect_timeout = None, keep_alive = 0):
         self.host = host
         kwargs = {}
         if user:
@@ -161,6 +184,7 @@ class ParamikoMachine(BaseRemoteMachine):
         if connect_timeout is not None:
             kwargs["timeout"] = connect_timeout
         self._client.connect(host, **kwargs)
+        self._keep_alive = keep_alive
         self._sftp = None
         BaseRemoteMachine.__init__(self, encoding, connect_timeout)
 
@@ -184,7 +208,9 @@ class ParamikoMachine(BaseRemoteMachine):
     @_setdoc(BaseRemoteMachine)
     def session(self, isatty = False, term = "vt100", width = 80, height = 24, new_session = False):
         # new_session is ignored for ParamikoMachine
-        chan = self._client.get_transport().open_session()
+        trans = self._client.get_transport()
+        trans.set_keepalive(self._keep_alive)
+        chan = trans.open_session()
         if isatty:
             chan.get_pty(term, width, height)
             chan.set_combine_stderr()
@@ -196,18 +222,18 @@ class ParamikoMachine(BaseRemoteMachine):
         return ShellSession(proc, self.encoding, isatty)
 
     @_setdoc(BaseRemoteMachine)
-    def popen(self, args, stdin = None, stdout = None, stderr = None, new_session = False):
+    def popen(self, args, stdin = None, stdout = None, stderr = None, new_session = False, cwd = None):
         # new_session is ignored for ParamikoMachine
         argv = []
         envdelta = self.env.getdelta()
-        argv.extend(["cd", str(self.cwd), "&&"])
+        argv.extend(["cd", str(cwd or self.cwd), "&&"])
         if envdelta:
             argv.append("env")
             argv.extend("%s=%s" % (k, v) for k, v in envdelta.items())
         argv.extend(args.formulate())
         cmdline = " ".join(argv)
         logger.debug(cmdline)
-        si, so, se = self._client.exec_command(cmdline, 1)
+        si, so, se = streams = self._client.exec_command(cmdline, 1)
         return ParamikoPopen(argv, si, so, se, self.encoding, stdin_file = stdin,
             stdout_file = stdout, stderr_file = stderr)
 
@@ -269,7 +295,9 @@ class ParamikoMachine(BaseRemoteMachine):
         if ipv6 and dhost == "localhost":
             dhost = "::1"
         srcaddr = ("::1", 0, 0, 0) if ipv6 else ("127.0.0.1", 0)
-        chan = self._client.get_transport().open_channel('direct-tcpip', (dhost, dport), srcaddr)
+        trans = self._client.get_transport()
+        trans.set_keepalive(self._keep_alive)
+        chan = trans.open_channel('direct-tcpip', (dhost, dport), srcaddr)
         return SocketCompatibleChannel(chan)
 
     #
@@ -289,6 +317,21 @@ class ParamikoMachine(BaseRemoteMachine):
         f = self.sftp.open(str(fn), 'wb')
         f.write(data)
         f.close()
+    def _path_stat(self, fn):
+        try:
+            st = self.sftp.stat(str(fn))
+        except IOError as e:
+            if e.errno == errno.ENOENT:
+                return None
+            raise OSError(e.errno)
+        res = StatRes((st.st_mode, 0, 0, 0, st.st_uid, st.st_gid,
+                       st.st_size, st.st_atime, st.st_mtime, 0))
+
+        if stat.S_ISDIR(st.st_mode):
+            res.text_mode = 'directory'
+        if stat.S_ISREG(st.st_mode):
+            res.text_mode = 'regular file'
+        return res
 
 
 
@@ -311,4 +354,39 @@ class SocketCompatibleChannel(object):
         return self._chan.recv(count)
 
 
+###################################################################################################
+# Custom iter_lines for paramiko.Channel
+###################################################################################################
+def _iter_lines(proc, decode, linesize):
 
+    try:
+        from selectors import DefaultSelector, EVENT_READ
+    except ImportError:
+        # Pre Python 3.4 implementation
+        from select import select
+        def selector():
+            while True:
+                rlist, _, _ = select([proc.stdout.channel], [], [])
+                for _ in rlist:
+                    yield
+    else:
+        # Python 3.4 implementation
+        def selector():
+            sel = DefaultSelector()
+            sel.register(proc.stdout.channel, EVENT_READ)
+            while True:
+                for key, mask in sel.select():
+                    yield
+
+    for _ in selector():
+        if proc.stdout.channel.recv_ready():
+            yield 0, decode(six.b(proc.stdout.readline(linesize)))
+        if proc.stdout.channel.recv_stderr_ready():
+            yield 1, decode(six.b(proc.stderr.readline(linesize)))
+        if proc.poll() is not None:
+            break
+
+    for line in proc.stdout:
+        yield 0, decode(six.b(line))
+    for line in proc.stderr:
+        yield 1, decode(six.b(line))
