@@ -74,16 +74,22 @@ class Path(object):
     def up(self, count = 1):
         """Go up in ``count`` directories (the default is 1)"""
         return self.join("../" * count)
-    def walk(self, filter = lambda p: True):  # @ReservedAssignment
+    def walk(self, filter = lambda p: True, dir_filter = lambda p: True):  # @ReservedAssignment
         """traverse all (recursive) sub-elements under this directory, that match the given filter.
         By default, the filter accepts everything; you can provide a custom filter function that
-        takes a path as an argument and returns a boolean"""
+        takes a path as an argument and returns a boolean
+        
+        :param filter: the filter (predicate function) for matching results. Only paths matching 
+                       this predicate are returned. Defaults to everything.
+        :param dir_filter: the filter (predicate function) for matching directories. Only directories
+                           matching this predicate are recursed into. Defaults to everything.
+        """
         for p in self.list():
             if filter(p):
                 yield p
-                if p.isdir():
-                    for p2 in p.walk(filter):
-                        yield p2
+            if p.isdir() and dir_filter(p):
+                for p2 in p.walk(filter, dir_filter):
+                    yield p2
 
     @property
     def basename(self):
@@ -93,6 +99,16 @@ class Path(object):
     def dirname(self):
         """The dirname component of this path"""
         raise NotImplementedError()
+        
+    @property
+    def suffix(self):
+        """The suffix of this file"""
+        raise NotImplementedError()
+    @property
+    def suffixes(self):
+        """This is a list of all suffixes"""
+        raise NotImplementedError()
+        
     @property
     def uid(self):
         """The user that owns this path. The returned value is a :class:`FSUser <plumbum.path.FSUser>`
@@ -128,6 +144,15 @@ class Path(object):
         raise NotImplementedError()
     def stat(self):
         raise NotImplementedError()
+    def with_name(self, name):
+        """Returns a path with the name replaced"""
+        raise NotImplementedError()
+    def with_suffix(self, suffix, depth=1):
+        """Returns a path with the suffix replaced. Up to last ``depth`` suffixes will be
+        replaces. None will replace all suffixes. If there are less than ``depth`` suffixes,
+        this will replace all suffixes. ``.tar.gz`` is an example where ``depth=2`` or
+        ``depth=None`` is useful"""
+        raise NotImplementedError()
     def glob(self, pattern):
         """Returns a (possibly empty) list of paths that matched the glob-pattern under this path"""
         raise NotImplementedError()
@@ -149,11 +174,13 @@ class Path(object):
     def open(self, mode = "r"):
         """opens this path as a file"""
         raise NotImplementedError()
-    def read(self):
-        """returns the contents of this file"""
+    def read(self, encoding=None):
+        """returns the contents of this file. By default the data is binary (``bytes``), but you can
+        specify the encoding, e.g., ``'latin1'`` or ``'utf8'``"""
         raise NotImplementedError()
-    def write(self, data):
-        """writes the given data to this file"""
+    def write(self, data, encoding=None):
+        """writes the given data to this file. By default the data is expected to be binary (``bytes``), 
+        but you can specify the encoding, e.g., ``'latin1'`` or ``'utf8'``"""
         raise NotImplementedError()
     def chown(self, owner = None, group = None, recursive = None):
         """Change ownership of this path.
@@ -246,7 +273,7 @@ class RelativePath(object):
     Therefore they are system agnostic (but closed under addition) 
     Paths are always absolute and point at "something", whether existent or not.
     
-    Relative paths are created by subtracting paths Path.relative_to
+    Relative paths are created by subtracting paths (``Path.relative_to``)
     """
     def __init__(self, parts):
         self.parts = parts
